﻿
DeclareModule ExtTools
  
  ; Инициализирует систему запуска внешних инструментов
  Declare InitExtTools(beforeMenuItem)
  
  ; Создает новый внешний инструмент
  Declare.l CreateExtTool(toolName$, fileName$, param$, shortcut = 0)
  
  ; Изменяет указанный внешний инструмент
  Declare.b ChangeExtTool(toolKey, toolName$, fileName$, param$, shortcut = 0)
  
  ; Удаляет указанный внешний инструмент
  Declare.b DeleteExtTool(toolKey)
  
  ; Делает текущим указанный по индексу внешний инструмент
  Declare.b SelectExtTool(toolKey)
  
  ; Инициализирует итерацию по списку внешних инструментов при помощи функции NextExtTool()
  Declare.b ExamineExtTools()
  
  ; Делает текущим очередной элемент списка внешних инструментов
  Declare.b NextExtTool()
  
  ; Возвращает индекс текущего элемента списка
  Declare.l ExtToolIndex()
  
  ; Возвращает для текущего элемента списка имя внешнего инструмента
  Declare.s ExtToolName()
  
  ; Возвращает для текущего элемента списка имя файла внешнего инструмента
  Declare.s ExtToolFileName()
  
  ; Возвращает для текущего элемента списка строку параметров внешнего инструмента
  Declare.s ExtToolParams()
  
  ; Возвращает для текущего элемента списка назначенные горячие клавиши
  Declare.l ExtToolShortcut()
  
  ; Возвращает для текущего элемента списка символьное отображение горячих клавиш
  Declare.s ExtToolShortcutString()
  
  ; Загружает сохраненные настройки внешних инструментов из файла
  Declare LoadExtTools(fileName$ = "")
  
  ; Сохраняет текущие настройки внешних инструментов в файл
  Declare SaveExtTools(fileName$ = "")

  ; Проверяет, не выбран ли один из пунктов меню внешних инструментов
  Declare.b IsExtToolMenuEvent(eventMenu)
  
  ; Запускает внешний инструмент
  Declare CallExtToolFunction(eventMenu)
  
EndDeclareModule


Module ExtTools
  ;Стартовое значение для идентификаторов пунктов меню, используемых в системе запуска внешних инструментов.
  ;Оно должно быть больше любых других идентификаторов пунктов меню, гаджетов и т.п.
  #ExtToolID_First = 9000
  #ExtToolID_Last = 9999
  Global toolMenuItemID = #ExtToolID_First
  
  Structure ExtToolFull
    Name.s
    Shortcut.i
    FileName.s
    Params.s
    
    menuItem.i
  EndStructure
  
  Global NewList lstTools.ExtToolFull()
  
  Structure ExtTool
    Name.s
    Shortcut.i
    FileName.s
    Params.s
  EndStructure
  
  Global NewList lstTmp.ExtTool()
  
  Global toolsFileName$ = GetCurrentDirectory() + "tools.dat"
  Global beforeItemID = -1, isChanged = #False
  
  ; Проверяет, инициализирована ли система запуска внешних инструментов.
  Procedure.b __IsInitExtTools()
    If beforeItemID = -1
      Debug "Инициализируйте систему запуска внешних инструментов при помощи функции InitExtTools()"
      ProcedureReturn #False
    EndIf
    
    ProcedureReturn #True
  EndProcedure
  
  ;===================================================================================================
  ; Инициализирует систему запуска внешних инструментов.
  ;
  ; beforeMenuItem - идентификатор пункта меню, перед которым будeт добавляться новые пункты для запуска внешних инструментов
  ;
  ; При при успешном завершении возвращается #True.
  ;===================================================================================================
  Procedure InitExtTools(beforeMenuItem)
    beforeItemID = beforeMenuItem
  EndProcedure
  
  ;===================================================================================================
  ; Создает новый внешний инструмент.
  ;
  ; toolName$ - имя внешнего инструмента для отображения в настройках
  ; fileName$ - полное имя запускаемого файла
  ; param$    - строка передаваемых параметров
  ; shortcut  - сочетание клавиш (#PB_Shortcut_*)
  ;
  ; Возвращает индекс внешнего инструмента в списке, который используется в остальных функциях модуля (параметр toolKey).
  ;===================================================================================================
  Procedure.l CreateExtTool(toolName$, fileName$, param$, shortcut = 0)
    Protected idx
    
    LastElement(lstTools())
    AddElement(lstTools())
    idx = ListIndex(lstTools())
    
    ChangeExtTool(idx, toolName$, fileName$, param$, shortcut)
    
    ProcedureReturn idx
  EndProcedure
  
  ;===================================================================================================
  ; Изменяет указанный внешний инструмент.
  ;
  ; toolKey   - индекс внешнего инструмента в списке
  ; toolName$ - имя внешнего инструмента для отображения в настройках
  ; fileName$ - полное имя запускаемого файла
  ; param$    - строка передаваемых параметров
  ; shortcut  - сочетание клавиш (#PB_Shortcut_*)
  ;
  ; При при успешном завершении возвращается #True.
  ;===================================================================================================
  Procedure.b ChangeExtTool(toolKey, toolName$, fileName$, param$, shortcut = 0)
    Protected tmp$, oldShortcut, menuInfo.MENUITEMINFO
    
    If Not SelectExtTool(toolKey)
      ProcedureReturn #False
    EndIf
    
    isChanged = #True
      
    lstTools()\Name = toolName$
    lstTools()\FileName = fileName$
    lstTools()\Params = param$
    
    If Not Hotkeys::IsInitHotkeys() Or Not __IsInitExtTools()
      ProcedureReturn #False
    EndIf
        
    If lstTools()\Shortcut <> shortcut And Not Hotkeys::FindHotkey(shortcut, #True)
      oldShortcut = lstTools()\Shortcut
      
      Hotkeys::UpdateHotkeyList(oldShortcut, shortcut)
      
      lstTools()\Shortcut = shortcut
      
      If lstTools()\menuItem And oldShortcut
        ;отвязываем предыдущее сочетание клавиш от пункта меню
        RemoveKeyboardShortcut(Hotkeys::winID, oldShortcut)
      EndIf
    EndIf
    
    If lstTools()\menuItem
      DeleteMenu_(MenuID(Hotkeys::menuID), lstTools()\menuItem, #MF_BYCOMMAND)
    Else
      lstTools()\menuItem = toolMenuItemID
      toolMenuItemID + 1
    EndIf
      
    tmp$ = lstTools()\Name
    If lstTools()\Shortcut
      tmp$ + Chr(9) + Hotkeys::GetShortcutString(lstTools()\Shortcut)
    EndIf
    
    With menuInfo
      \cbSize = SizeOf(MENUITEMINFO)
      \fMask = #MIIM_TYPE | #MIIM_ID
      \fType = #MFT_STRING
      \dwTypeData = @tmp$
      \wID = lstTools()\menuItem
    EndWith
    
    InsertMenuItem_(MenuID(Hotkeys::menuID), beforeItemID, #False, menuInfo)    
    
    If lstTools()\Shortcut
      ;привязываем сочетание клавиш к пункту меню
      AddKeyboardShortcut(Hotkeys::winID, lstTools()\Shortcut, lstTools()\menuItem)
    EndIf
    
    ProcedureReturn #True
  EndProcedure

  ;===================================================================================================
  ; Удаляет указанный внешний инструмент.
  ;
  ; toolKey - индекс внешнего инструмента в списке
  ;
  ; При при успешном завершении возвращается #True.
  ;===================================================================================================
  Procedure.b DeleteExtTool(toolKey)
    If Not SelectExtTool(toolKey)
      ProcedureReturn #False
    EndIf
    
    isChanged = #True
    
    If lstTools()\menuItem
      DeleteMenu_(MenuID(Hotkeys::menuID), lstTools()\menuItem, #MF_BYCOMMAND)
    EndIf
    
    Hotkeys::UpdateHotkeyList(lstTools()\Shortcut)
    DeleteElement(lstTools())
    
    ProcedureReturn #True
  EndProcedure
  
  ;===================================================================================================
  ; Делает текущим указанный по индексу внешний инструмент.
  ; При удачном завершении возвращается #True.
  ;===================================================================================================
  Procedure.b SelectExtTool(toolKey)
    ProcedureReturn Bool(SelectElement(lstTools(), toolKey))
  EndProcedure

  ;===================================================================================================
  ; Инициализирует итерацию по списку внешних инструментов при помощи функции NextExtTool().
  ; Если список действий не пуст, то возвращается #True.
  ;===================================================================================================
  Procedure.b ExamineExtTools()
    ResetList(lstTools())
    ProcedureReturn Bool(ListSize(lstTools()))
  EndProcedure
  
  ;===================================================================================================
  ; Делает текущим очередной элемент списка внешних инструментов.
  ; При удачном завершении возвращается #True. Эта функция делает возможным итерацию по списку внешних инструментов.
  ; Для перемещения указателя в начало списка используйте функцию ExamineExtTools().
  ;===================================================================================================
  Procedure.b NextExtTool()
    ProcedureReturn Bool(NextElement(lstTools()))
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает индекс текущего элемента списка.
  ; Если текущий элемент не установлен, то возвращается -1.
  ;===================================================================================================
  Procedure.l ExtToolIndex()
    ProcedureReturn ListIndex(lstTools())
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка имя внешнего инструмента.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ExtToolName()
    If ListIndex(lstTools()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn lstTools()\Name
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка имя файла внешнего инструмента.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ExtToolFileName()
    If ListIndex(lstTools()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn lstTools()\FileName
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка строку параметров внешнего инструмента.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ExtToolParams()
    If ListIndex(lstTools()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn lstTools()\Params
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка назначенные горячие клавиши.
  ; Если текущий элемент не установлен, то возвращается 0.
  ;===================================================================================================
  Procedure.l ExtToolShortcut()
    If ListIndex(lstTools()) = -1
      ProcedureReturn 0
    Else
      ProcedureReturn lstTools()\Shortcut
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает для текущего элемента списка символьное отображение горячих клавиш.
  ; Если текущий элемент не установлен, то возвращается пустая строка.
  ;===================================================================================================
  Procedure.s ExtToolShortcutString()
    If ListIndex(lstTools()) = -1
      ProcedureReturn ""
    Else
      ProcedureReturn Hotkeys::GetShortcutString(lstTools()\Shortcut)
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Загружает сохраненные настройки внешних инструментов из файла.
  ;===================================================================================================
  Procedure LoadExtTools(fileName$ = "")
    If FileSize(fileName$) < 0 
      fileName$ = toolsFileName$
    EndIf
    
    If ParseJSON(0, Hotkeys::EncodedFileToJSONString(fileName$)) And JSONType(JSONValue(0)) = #PB_JSON_Array
      ExtractJSONList(JSONValue(0), lstTmp())
      FreeJSON(0)
  
      ForEach lstTmp()
        CreateExtTool(lstTmp()\Name, lstTmp()\FileName, lstTmp()\Params, lstTmp()\Shortcut)
      Next
      
      ClearList(lstTmp())
      
      isChanged = #False
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Сохраняет текущие настройки внешних инструментов в файл.
  ;===================================================================================================
  Procedure SaveExtTools(fileName$ = "")
    If isChanged
      If fileName$ = ""
        fileName$ = toolsFileName$
      EndIf
      
      ForEach lstTools()
        AddElement(lstTmp())
        lstTmp()\Name     = lstTools()\Name
        lstTmp()\FileName = lstTools()\FileName
        lstTmp()\Params   = lstTools()\Params
        lstTmp()\Shortcut = lstTools()\Shortcut
      Next
      
      If CreateJSON(0)
        InsertJSONList(JSONValue(0), lstTmp())
        Hotkeys::JSONStringToEncodedFile(fileName$, ComposeJSON(0))
        FreeJSON(0)
      EndIf
      
      ClearList(lstTmp())
      
      isChanged = #False
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Проверяет, не выбран ли один из пунктов меню внешних инструментов.
  ;===================================================================================================
  Procedure.b IsExtToolMenuEvent(eventMenu)
    ProcedureReturn Bool(eventMenu >= #ExtToolID_First And eventMenu <= #ExtToolID_Last)
  EndProcedure
  
  ;===================================================================================================
  ; Запускает внешний инструмент.
  ;===================================================================================================
  Procedure CallExtToolFunction(eventMenu)
    PushListPosition(lstTools())
    ForEach lstTools()
      If eventMenu = lstTools()\menuItem
        RunProgram(lstTools()\FileName, lstTools()\Params, "")
      EndIf
    Next
    PopListPosition(lstTools())
  EndProcedure
  
EndModule

; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 219
; FirstLine = 215
; Folding = ----
; EnableXP