﻿;===================================================================================================
; Этот модуль является вспомогательным для всех модулей, которые регистрируют в программе сочетания
; горячих клавиш (Actions, ExtTools).
;===================================================================================================
DeclareModule Hotkeys
  Global winID = -1, menuID
  
  ; Инициализирует систему поддержки горячих клавиш
  Declare.b InitHotkeys(mainWinID, mainMenuID)
  
  ; Проверяет, инициализирована ли система поддержки горячих клавиш
  Declare.b IsInitHotkeys()
  
  ; Возвращает символьное отображение указанного сочетания клавиш
  Declare.s GetShortcutString(shortcut)
  
  ; Ищет указанное сочетание горячих клавиш в списке
  Declare.b FindHotkey(hotkey, showMsg = #False)
  
  ; Актуализирует список горячих клавиш при изменении
  Declare UpdateHotkeyList(oldHotkey, newHotkey = 0)
  
  ; Возвращает расшифрованное содержимое файла, созданного с помощью функции JSONStringToEncodedFile()
  Declare.s EncodedFileToJSONString(fileName$)
    
  ; Записывает содержимое JSON-строки в зашифрованный файл
  Declare JSONStringToEncodedFile(fileName$, json$)
    
EndDeclareModule


Module Hotkeys
  ; Глобальный список всех сочетаний горячих клавиш, используемых в программе
  Global NewMap mapHotkeys()
  
  ;===================================================================================================
  ; Инициализирует систему поддержки горячих клавиш.
  ;
  ; mainWinID  - идентификатор главного окна приложения
  ; mainMenuID - идентификатор главного меню приложения
  ;
  ; При при успешном завершении возвращается #True.
  ;===================================================================================================
  Procedure.b InitHotkeys(mainWinID, mainMenuID)
    Protected res.b = Bool(IsWindow(mainWinID) And IsMenu(mainMenuID))
    
    If res
      winID = mainWinID
      menuID = mainMenuID
    EndIf
    
    ProcedureReturn res
  EndProcedure
  
  ;===================================================================================================
  ; Проверяет, инициализирована ли система поддержки горячих клавиш.
  ;===================================================================================================
  Procedure.b IsInitHotkeys()
    If winID = -1
      Debug "Инициализируйте систему поддержки горячих клавиш при помощи функции InitHotkeys()"
      ProcedureReturn #False
    EndIf
    
    ProcedureReturn #True
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает символьное отображение указанного сочетания клавиш.
  ;===================================================================================================
  Procedure.s GetShortcutString(shortcut)
    Protected res$ = ""
    
    If shortcut
      If #PB_Shortcut_Control & shortcut
        res$ + "Ctrl+"
        shortcut - #PB_Shortcut_Control
      EndIf
      If #PB_Shortcut_Alt & shortcut
        res$ + "Alt+"
        shortcut - #PB_Shortcut_Alt
      EndIf
      If #PB_Shortcut_Shift & shortcut
        res$ + "Shift+"
        shortcut - #PB_Shortcut_Shift
      EndIf
      
      Select shortcut
        Case #PB_Shortcut_Back                      : res$ + "Backspace"
        Case #PB_Shortcut_Tab                       : res$ + "Tab"
        Case #PB_Shortcut_Clear                     : res$ + "Num 5"
        Case #PB_Shortcut_Return                    : res$ + "Enter"
        Case #PB_Shortcut_Menu                      : res$ + "Menu"
        Case #PB_Shortcut_Pause                     : res$ + "Pause"
        Case #PB_Shortcut_Capital                   : res$ + "CapsLock"
        Case #PB_Shortcut_Escape                    : res$ + "Esc"
        Case #PB_Shortcut_Space                     : res$ + "Space"
        Case #PB_Shortcut_PageUp                    : res$ + "PageUp"
        Case #PB_Shortcut_PageDown                  : res$ + "PageDown"
        Case #PB_Shortcut_End                       : res$ + "End"
        Case #PB_Shortcut_Home                      : res$ + "Home"
        Case #PB_Shortcut_Left                      : res$ + "Left"
        Case #PB_Shortcut_Up                        : res$ + "Up"
        Case #PB_Shortcut_Right                     : res$ + "Right"
        Case #PB_Shortcut_Down                      : res$ + "Down"
        Case #PB_Shortcut_Select                    : res$ + "Select"
        Case #PB_Shortcut_Print                     : res$ + "Print"
        Case #PB_Shortcut_Execute                   : res$ + "Execute"
        Case #PB_Shortcut_Snapshot                  : res$ + "PrintScreen"
        Case #PB_Shortcut_Insert                    : res$ + "Insert"
        Case #PB_Shortcut_Delete                    : res$ + "Delete"
        Case #PB_Shortcut_Help                      : res$ + "Help"
        Case #PB_Shortcut_0 To #PB_Shortcut_9       : res$ + Chr(shortcut)
        Case #PB_Shortcut_A To #PB_Shortcut_Z       : res$ + Chr(shortcut)
        Case #PB_Shortcut_LeftWindows               : res$ + "LWin"
        Case #PB_Shortcut_RightWindows              : res$ + "RWin"
        Case #PB_Shortcut_Apps                      : res$ + "Apps"
        Case #PB_Shortcut_Pad0 To #PB_Shortcut_Pad9 : res$ + "Num " + Str(shortcut - #PB_Shortcut_Pad0)
        Case #PB_Shortcut_Multiply                  : res$ + "Num *"
        Case #PB_Shortcut_Add                       : res$ + "Num +"
        Case #PB_Shortcut_Separator                 : res$ + "Separator"
        Case #PB_Shortcut_Subtract                  : res$ + "Num -"
        Case #PB_Shortcut_Decimal                   : res$ + "Num Del"
        Case #PB_Shortcut_Divide                    : res$ + "Num /"
        Case #PB_Shortcut_F1 To #PB_Shortcut_F24    : res$ + "F" + Str(shortcut - #PB_Shortcut_F1 + 1)
        Case #PB_Shortcut_Numlock                   : res$ + "NumLock"
        Case #PB_Shortcut_Scroll                    : res$ + "ScrollLock"
        Case 186                                    : res$ + ";"
        Case 187                                    : res$ + "="
        Case 188                                    : res$ + ","
        Case 189                                    : res$ + "-"
        Case 190                                    : res$ + "."
        Case 191                                    : res$ + "/"
        Case 192                                    : res$ + "~"
        Case 219                                    : res$ + "["
        Case 220                                    : res$ + "\"
        Case 221                                    : res$ + "]"
        Case 222                                    : res$ + "'"
        Default                                     : res$ + ""
      EndSelect
    EndIf
    
    ProcedureReturn res$
  EndProcedure
  
  ;===================================================================================================
  ; Ищет указанное сочетание горячих клавиш в списке
  ;===================================================================================================
  Procedure.b FindHotkey(hotkey, showMsg = #False)
    Protected res = Bool(hotkey <> 0 And FindMapElement(mapHotkeys(), Str(hotkey)))
    
    If res And showMsg
      MessageRequester("Warning", "Сочетание клавиш " + GetShortcutString(hotkey) + " уже используется!", #PB_MessageRequester_Warning)
    EndIf
    
    ProcedureReturn res
  EndProcedure
  
  ;===================================================================================================
  ; Актуализирует список горячих клавиш при изменении
  ;===================================================================================================
  Procedure UpdateHotkeyList(oldHotkey, newHotkey = 0)
    If newHotkey
      If oldHotkey
        DeleteMapElement(mapHotkeys(), Str(oldHotkey))
      EndIf
      mapHotkeys(Str(newHotkey)) = 0
    Else
      DeleteMapElement(mapHotkeys(), Str(oldHotkey))
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает расшифрованное содержимое файла, созданного с помощью функции JSONStringToEncodedFile()
  ;===================================================================================================
  Procedure.s EncodedFileToJSONString(fileName$)
    Protected encoded$, decoded$, bufIn, bufOut
    
    If ReadFile(0, fileName$)
      bufIn = Lof(0) + 5
      encoded$ = Space(bufIn)  
      
      ReadData(0, @encoded$, bufIn)
      CloseFile(0)
      
      bufOut = bufIn * 1.35
      decoded$ = Space(bufOut)
      
      Base64Decoder(@encoded$, bufIn, @decoded$, bufOut)
      
      ProcedureReturn decoded$
    Else
      ProcedureReturn ""
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Записывает содержимое JSON-строки в зашифрованный файл
  ;===================================================================================================
  Procedure JSONStringToEncodedFile(fileName$, json$)
    Protected encoded$, bufIn, bufOut
      
    bufIn = StringByteLength(json$) + 5
    bufOut = bufIn * 1.35
    encoded$ = Space(bufOut)
    
    If CreateFile(0, fileName$)
      WriteData(0, @encoded$, Base64Encoder(@json$, bufIn, @encoded$, bufOut))
      CloseFile(0)
    EndIf
  EndProcedure

EndModule

; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 143
; FirstLine = 137
; Folding = --
; EnableXP