﻿
DeclareModule Lang
  ; Создает новый строковый ресурс или устанавливает значение существующего
  Declare SetLangRes(key.i, str$)
  
  ; Возвращает строковое значение, соответствующее указанному ключу
  Declare.s GetLangRes(key.i)
  
  ; Загружает язык интерфейса из указанного файла
  Declare LoadLanguage(fileName$)
  
  ; Сохраняет текущий язык интерфейса в файл default.lng
  Declare SaveLanguageDefault(langDir$)
EndDeclareModule


Module Lang
  Global NewMap mapLang.s()
  
  ;===================================================================================================
  ; Создает новый строковый ресурс или устанавливает значение существующего.
  ;
  ; key  - числовой идентификатор ресурса
  ; str$ - устанавливаемое строковое значение ресурса
  ;===================================================================================================
  Procedure SetLangRes(key.i, str$)
    Protected key$ = Str(key)
    
    If FindMapElement(mapLang(), key$)
      mapLang() = str$
    Else
      mapLang(key$) = str$
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Возвращает строковое значение, соответствующее указанному ключу.
  ;===================================================================================================
  Procedure.s GetLangRes(key.i)
    Protected key$ = Str(key)
    
    If FindMapElement(mapLang(), key$)
      ProcedureReturn mapLang(key$)
    Else
      Debug "Ключ " + key$ + " в mapLang() не определен!"
      ProcedureReturn ""
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Загружает язык интерфейса из указанного файла.
  ;===================================================================================================
  Procedure LoadLanguage(fileName$)
    Protected key$, str$
    
    If LCase(GetFilePart(fileName$, #PB_FileSystem_NoExtension)) = "default" : ProcedureReturn : EndIf
    
    If OpenPreferences(fileName$, #PB_Preference_NoSpace | #PB_Preference_GroupSeparator)
      
      PreferenceGroup("")
      ExaminePreferenceKeys()
      While NextPreferenceKey()
        key$ = PreferenceKeyName()
        str$ = ReadPreferenceString(key$, "")
        If FindMapElement(mapLang(), key$) And str$ <> ""
          mapLang() = str$
        EndIf
      Wend
      
      ClosePreferences()
    EndIf
  EndProcedure
  
  ;===================================================================================================
  ; Сохраняет текущий язык интерфейса в файл default.lng.
  ; Эта процедура применяется в служебных целях для автоматического формирования языкового файла
  ; и не должна использоваться в готовой программе.
  ;===================================================================================================
  Procedure SaveLanguageDefault(langDir$)
    If FileSize(langDir$) = -2
      Protected fileName$ = langDir$ + "default.lng"
      
      DeleteFile(fileName$)
      
      OpenPreferences(fileName$, #PB_Preference_NoSpace | #PB_Preference_GroupSeparator)
      
      PreferenceGroup("")
      ForEach mapLang()
        WritePreferenceString(MapKey(mapLang()), mapLang())
      Next
      
      ClosePreferences()
    EndIf
  EndProcedure
EndModule

; IDE Options = PureBasic 5.51 (Windows - x86)
; Folding = --
; EnableXP