﻿
Structure MDIWindow
  Child.i
EndStructure

Global NewMap mapMDIWindow.MDIWindow()

;===================================================================================================
; Включает/отключает определенные пункты меню и кнопки тулбара в зависимости от текущего количества MDI-окон.
;===================================================================================================
Procedure __DisableMDIMenus()
  mode = Bool(MapSize(mapMDIWindow()) < 1)
  DisableMenuItem(#MainMenu, #Menu_Close, mode)
  DisableMenuItem(#MainMenu, #Menu_CloseAll, mode)
  
  mode = Bool(MapSize(mapMDIWindow()) < 2)
  DisableMenuItem(#MainMenu, #Menu_Previous, mode)
  DisableMenuItem(#MainMenu, #Menu_Next, mode)
  DisableMenuItem(#MainMenu, #Menu_TileV, mode)
  DisableMenuItem(#MainMenu, #Menu_TileH, mode)
  DisableMenuItem(#MainMenu, #Menu_Cascade, mode)
  
  DisableToolBarButton(#MainToolbar, #Menu_Previous, mode)
  DisableToolBarButton(#MainToolbar, #Menu_Next, mode)
EndProcedure

;===================================================================================================
; Проверяет, есть ли указанное MDI-окно в карте окон.
;===================================================================================================
Procedure.i FindMDIWindow(window)
  ProcedureReturn FindMapElement(mapMDIWindow(), Str(window))
EndProcedure

;===================================================================================================
; Открывает новое MDI-окно.
;===================================================================================================
Procedure OpenMDIWindow()
  Protected key$ = Str(AddGadgetItem(#MDIGadget, -1, "MDI Window " + MapSize(mapMDIWindow()), 0, #PB_Window_MaximizeGadget | #PB_Window_Maximize))
  
  AddMapElement(mapMDIWindow(), key$)
  mapMDIWindow()\Child = 0  ;сюда будет записываться ID дочернего гаджета
  
  __DisableMDIMenus()
EndProcedure

;===================================================================================================
; Закрывает указанное MDI-окно.
;===================================================================================================
Procedure CloseMDIWindow(window)
  If FindMDIWindow(window)
    CloseWindow(window)
    DeleteMapElement(mapMDIWindow(), Str(window))   
  
    __DisableMDIMenus()
  EndIf
EndProcedure

;===================================================================================================
; Закрывает текущее MDI-окно.
;===================================================================================================
Procedure CloseCurrentMDIWindow()
  Protected window = GetGadgetState(#MDIGadget)
  
  If window <> -1
    CloseMDIWindow(window)
  EndIf
EndProcedure
  
;===================================================================================================
; Закрывает все MDI-окна.
;===================================================================================================
Procedure CloseAllMDIWindows()
  ForEach mapMDIWindow()
    CloseWindow(Val(MapKey(mapMDIWindow())))
  Next
  ClearMap(mapMDIWindow())
  
  __DisableMDIMenus()
EndProcedure

;===================================================================================================
; Располагает MDI-окна по вертикали.
;===================================================================================================
Procedure ArrangeMDIVertically()
  SetGadgetState(#MDIGadget, #PB_MDI_TileVertically)
EndProcedure

;===================================================================================================
; Располагает MDI-окна по горизонтали.
;===================================================================================================
Procedure ArrangeMDIHorizontally()
  SetGadgetState(#MDIGadget, #PB_MDI_TileHorizontally)
EndProcedure

;===================================================================================================
; Располагает MDI-окна каскадом.
;===================================================================================================
Procedure ArrangeMDICascade()
  SetGadgetState(#MDIGadget, #PB_MDI_Cascade)
EndProcedure

;===================================================================================================
; Делает активным предыдущее MDI-окно.
;===================================================================================================
Procedure PreviousMDIWindow()
  SetGadgetState(#MDIGadget, #PB_MDI_Previous)
EndProcedure

;===================================================================================================
; Делает активным следующее MDI-окно.
;===================================================================================================
Procedure NextMDIWindow()
  SetGadgetState(#MDIGadget, #PB_MDI_Next)
EndProcedure


; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 50
; FirstLine = 20
; Folding = --
; EnableXP