﻿
Global mutexName.s = "Application_Mutex"  ;Внимание! Эта строка должна быть уникальна для каждого приложения

Global dataDir$ = GetCurrentDirectory() + "Data\"
Global langDir$ = GetCurrentDirectory() + "Lang\"

Global shortcutsFile$ = dataDir$ + "shortcuts.dat"

Global lastPrefTreeItem = 0

Global isQuit.b = #False    ;эта переменная принимает значение #True перед выходом из программы
Global isHidden.b = #False  ;эта переменная принимает значение #True при сворачивании приложения в трей и #False - при разворачивании

Declare SetOptionsDefault()
Declare SetLanguageDefault()
Declare CreateMenuActions()
Declare CheckOneInstance()

;подключаем модули
XIncludeFile "Options.pbi"
XIncludeFile "Plugins.pbi"
XIncludeFile "Lang.pbi"

XIncludeFile "Hotkeys.pbi"
XIncludeFile "Actions.pbi"
XIncludeFile "ExtTools.pbi"

UseModule Options
UseModule Actions
UseModule Lang

CompilerIf Defined(ExtTools, #PB_Module)
  Global isCreatedTool.b  ;эта переменная принимает значение #True при добавлении внешнего инструмента или #False - при редактировании
CompilerEndIf

XIncludeFile "Constants.pbi"

CompilerSelect #AppMode
  CompilerCase #AppMode_TDI
    XIncludeFile "TDI.pbi"
    
  CompilerCase #AppMode_MDI
    XIncludeFile "MDI.pbi"
CompilerEndSelect

XIncludeFile "WinAbout.pbi"
XIncludeFile "WinMain.pbi"
CompilerIf Defined(ExtTools, #PB_Module)
  XIncludeFile "WinTool.pbi"
CompilerEndIf
XIncludeFile "WinPref.pbi"

;===================================================================================================

SetOptionsDefault()   ;устанавливаем настройки по умолчанию 
LoadOptions()         ;загружаем сохраненные настройки

CheckOneInstance()    ;проверяем на запуск более одной копии приложения

SetLanguageDefault()  ;формируем строковые ресурсы для языка по умолчанию
; SaveLanguageDefault(langDir$)

ParseCommandLine()    ;анализируем параметры командной строки

LoadLanguage(langDir$ + GetOptString("language") + ".lng")  ;загружаем установленный язык интерфейса

CompilerIf Defined(Plugins, #PB_Module)
  Global plugDir$ = GetCurrentDirectory() + "Plugins\"
  Plugins::LoadPlugins(plugDir$) ;загружаем доступные плагины
CompilerEndIf

;корректируем размеры и положение главного окна
ExamineDesktops()
x = GetOptQuad("x") : y = GetOptQuad("y") : w = GetOptQuad("w") : h = GetOptQuad("h")
If w < 800 Or w > DesktopWidth(0)   : SetOptQuad("w", 800)  : EndIf
If h < 600 Or h > DesktopHeight(0)  : SetOptQuad("h", 600)  : EndIf
If x < 0 Or (x + w) > DesktopWidth(0)   : SetOptQuad("x", (DesktopWidth(0) - w) / 2)  : EndIf
If y < 0 Or (y + h) > DesktopHeight(0)  : SetOptQuad("y", (DesktopHeight(0) - h) / 2) : EndIf

;открываем главное окно приложения
OpenWinMain(x, y, w, h)
If GetOptQuad("state") = #PB_Window_Maximize
  SetWindowState(#WinMain, #PB_Window_Maximize)
EndIf
SetProp_(WindowID(#WinMain), @mutexName, 1)
If GetOptQuad("showTrayIcon") And GetOptQuad("minimizeOnStart")
  MinimizeToTray()
EndIf

CreateMenuActions() ;создаем необходимые действия и привязываем их к пунктам меню

CompilerIf Defined(ExtTools, #PB_Module)
  Global toolsFile$ = dataDir$ + "tools.dat"
  ExtTools::InitExtTools(#Menu_ToolsPref)
  ExtTools::LoadExtTools(toolsFile$)  ;загружаем сохраненные настройки внешних инструментов
CompilerEndIf

Repeat
  event = WaitWindowEvent()
  
  Select EventWindow()
    Case #WinPref   : WinPref_Events(event)
    Case #WinTool   : WinTool_Events(event)
    Case #WinAbout  : WinAbout_Events(event)
    Default         : WinMain_Events(event)
  EndSelect
Until isQuit = #True

CompilerIf Defined(Plugins, #PB_Module)
  Plugins::CloseAllPlugins()   ;выгружаем плагины
CompilerEndIf

;сохраняем текущие размеры и положение главного окна
If GetWindowState(#WinMain) = #PB_Window_Normal
  SetOptQuad("x", WindowX(#WinMain))
  SetOptQuad("y", WindowY(#WinMain))
  SetOptQuad("w", WindowWidth(#WinMain))
  SetOptQuad("h", WindowHeight(#WinMain))
EndIf
SetOptQuad("state", GetWindowState(#WinMain))

SaveOptions()                         ;сохраняем основные настройки
SaveActionShortcuts(shortcutsFile$)   ;сохраняем настройки горячих клавиш
CompilerIf Defined(ExtTools, #PB_Module)
  ExtTools::SaveExtTools(toolsFile$)  ;сохраняем настройки внешних инструментов
CompilerEndIf

;===================================================================================================
; Устанавливает значения настроек по умолчанию
;===================================================================================================
Procedure SetOptionsDefault()
  SetOptQuad("x", 0, "General")
  SetOptQuad("y", 0, "General")
  SetOptQuad("w", 800, "General")
  SetOptQuad("h", 600, "General")
  SetOptQuad("state", #PB_Window_Normal, "General")
  SetOptString("language", #LangDefault, "General")
  SetOptQuad("showTrayIcon", #True, "General")
  SetOptQuad("minimizeOnStart", #False, "General")
  SetOptQuad("oneInstance", #True, "General")
EndProcedure

;===================================================================================================
; Формирует строковые ресурсы для языка по умолчанию
;===================================================================================================
Procedure SetLanguageDefault()
  SetLangRes(1, "Load the specified language")
  SetLangRes(2, "This help window")
  SetLangRes(3, "Supported command-line options:")
  SetLangRes(4, "Name")
  SetLangRes(5, "Version")
  SetLangRes(6, "Description")
  SetLangRes(7, "Action")
  SetLangRes(8, "Shortcuts")
  
  CompilerIf Defined(ExtTools, #PB_Module)
    SetLangRes(9, "External tool")
    SetLangRes(10, "Select file")
  CompilerEndIf
  
  SetLangRes(#Menu_File, "File") 
  SetLangRes(#Menu_Pref, "Preferences")
  SetLangRes(#Menu_Quit, "Exit")
  SetLangRes(#Menu_ShowMainForm, "Show main window")
  SetLangRes(#Menu_Help, "Help")
  SetLangRes(#Menu_About, "About")
  SetLangRes(#Menu_Window, "Window")
  SetLangRes(#PrefBtn_OK, "OK")
  SetLangRes(#PrefBtn_Cancel, "Cancel")
  SetLangRes(#PrefBtn_Apply, "Apply")
  SetLangRes(#PrefFrm_Shortcuts, "Shortcuts")
  SetLangRes(#PrefFrm_Plugins, "Plugins")
  SetLangRes(#PrefFrm_General, "General")
  SetLangRes(#PrefTxt_Language, "Language")
  SetLangRes(#PrefChB_ShowTrayIcon, "Minimize to system tray")
  SetLangRes(#PrefChB_MinimizeOnStart, "Minimize to system tray on start")
  SetLangRes(#PrefChB_OneInstance, "Allow to run single instance only")
  SetLangRes(#PrefTxt_AfterRestart, "will change only after restart")
  SetLangRes(#PrefTxt_Shortcut, "Select new shortcut:")
  SetLangRes(#PrefBtn_Assign, "Set shortcut")
  
  CompilerIf Defined(ExtTools, #PB_Module)
    SetLangRes(#PrefFrm_ExtTools, "External tools")
    SetLangRes(#Menu_Tools, "Tools")
    SetLangRes(#Menu_ToolsPref, "Customize tools")
    SetLangRes(#PrefBtn_AddTool, "Add")
    SetLangRes(#PrefBtn_EditTool, "Edit")
    SetLangRes(#PrefBtn_DelTool, "Delete")
    SetLangRes(#ToolTxt_ToolName, "Name")
    SetLangRes(#ToolTxt_ToolFile, "File name")
    SetLangRes(#ToolTxt_ToolParam, "Parameters")
    SetLangRes(#ToolTxt_Shortcut, "Hotkeys")
    SetLangRes(#ToolBtn_Select, "Select")
  CompilerEndIf
  
  CompilerIf Defined(Plugins, #PB_Module)
    SetLangRes(#Menu_Plugins, "Plugins")
  CompilerEndIf
  
  CompilerIf #AppMode = #AppMode_TDI Or #AppMode = #AppMode_MDI
    SetLangRes(#Menu_New, "New")
    SetLangRes(#Menu_Close, "Close")
    SetLangRes(#Menu_CloseAll, "Close all")
    SetLangRes(#Menu_Previous, "Previous window")
    SetLangRes(#Menu_Next, "Next window")
    
    CompilerIf #AppMode = #AppMode_MDI
      SetLangRes(#Menu_TileV, "Tile vertically")
      SetLangRes(#Menu_TileH, "Tile horizontally")
      SetLangRes(#Menu_Cascade, "Cascade")
    CompilerEndIf      
  CompilerEndIf      
EndProcedure


;===================================================================================================
; Создает необходимые действия и привязывает их к пунктам меню
;===================================================================================================
Procedure CreateMenuActions()
  Hotkeys::InitHotkeys(#WinMain, #MainMenu)

  CompilerSelect #AppMode
    CompilerCase #AppMode_TDI
      actNew = CreateAction(@OpenTDIPanel(), #PB_Shortcut_Control | #PB_Shortcut_N, GetLangRes(#Menu_New))
      actClose = CreateAction(@CloseTDIPanel(), #PB_Shortcut_Control | #PB_Shortcut_W, GetLangRes(#Menu_Close))
      actCloseAll = CreateAction(@CloseAllTDIPanels(), 0, GetLangRes(#Menu_CloseAll))
      actNext = CreateAction(@NextTDIPanel(), #PB_Shortcut_Control | #PB_Shortcut_Tab, GetLangRes(#Menu_Next))
      actPrevious = CreateAction(@PreviousTDIPanel(), #PB_Shortcut_Shift | #PB_Shortcut_Control | #PB_Shortcut_Tab, GetLangRes(#Menu_Previous))
      
    CompilerCase #AppMode_MDI
      actNew = CreateAction(@OpenMDIWindow(), #PB_Shortcut_Control | #PB_Shortcut_N, GetLangRes(#Menu_New))
      actClose = CreateAction(@CloseCurrentMDIWindow(), #PB_Shortcut_Control | #PB_Shortcut_W, GetLangRes(#Menu_Close))
      actCloseAll = CreateAction(@CloseAllMDIWindows(), 0, GetLangRes(#Menu_CloseAll))
      actNext = CreateAction(@NextMDIWindow(), #PB_Shortcut_Control | #PB_Shortcut_Tab, GetLangRes(#Menu_Next))
      actPrevious = CreateAction(@PreviousMDIWindow(), #PB_Shortcut_Shift | #PB_Shortcut_Control | #PB_Shortcut_Tab, GetLangRes(#Menu_Previous))
      
      actTileHorMDI = CreateAction(@ArrangeMDIHorizontally(), 0, GetLangRes(#Menu_TileH))
      actTileVerMDI = CreateAction(@ArrangeMDIVertically(), 0, GetLangRes(#Menu_TileV))
      actCascadeMDI = CreateAction(@ArrangeMDICascade(), 0, GetLangRes(#Menu_Cascade))
  CompilerEndSelect
  actPref = CreateAction(@OpenWinPref(), #PB_Shortcut_F9, GetLangRes(#Menu_Pref))
  actHelp = CreateAction(@OpenWinAbout(), #PB_Shortcut_F1, GetLangRes(#Menu_About))
  actQuit = CreateAction(@ExitApp(), 0, GetLangRes(#Menu_Quit))
  
  LoadActionShortcuts(shortcutsFile$) ;загружаем сохраненные ранее настройки горячих клавиш

  CompilerIf #AppMode = #AppMode_TDI Or #AppMode = #AppMode_MDI
    BindMenuAction(#Menu_New, actNew)
    BindMenuAction(#Menu_Close, actClose)
    BindMenuAction(#Menu_CloseAll, actCloseAll)
    BindMenuAction(#Menu_Next, actNext)
    BindMenuAction(#Menu_Previous, actPrevious)
    CompilerIf #AppMode = #AppMode_MDI
      BindMenuAction(#Menu_TileH, actTileHorMDI)
      BindMenuAction(#Menu_TileV, actTileVerMDI)
      BindMenuAction(#Menu_Cascade, actCascadeMDI)
    CompilerEndIf        
  CompilerEndIf      
  BindMenuAction(#Menu_Pref, actPref)
  BindMenuAction(#Menu_Quit, actQuit)
  BindMenuAction(#Menu_About, actHelp)
EndProcedure


;===================================================================================================
; Проверяет, не запущено ли приложение дважды
;===================================================================================================
Procedure CheckOneInstance()
  mutex = CreateMutex_(#Null, #True, @mutexName)
  If GetLastError_() = #ERROR_ALREADY_EXISTS And GetOptQuad("oneInstance")
    hWnd.i = GetWindow_(GetDesktopWindow_(), #GW_CHILD)
    While hWnd > 0
      If GetProp_(hWnd, @mutexName) = 1
        If IsIconic_(hWnd) > 0
          ShowWindowAsync_(hWnd, #SW_RESTORE)
        EndIf
        SetForegroundWindow_(hWnd)
        Break
      EndIf
      hWnd = GetWindow_(hWnd, #GW_HWNDNEXT)
    Wend
    CloseHandle_(mutex)
    End
  EndIf
  ReleaseMutex_(mutex)
EndProcedure

;===================================================================================================

;-TODO
; не давать перемещать окно за пределы рабочего стола
; возможность отключать загрузку отдельных плагинов
; поддержка Drag&Drop файлов на главное окно
; SplashScreen
; скрываемые боковые панели

;-What's new
; v0.9.5
; - В модуль Actions добавлена возможность итерации по списку действий в стиле PureBasic при помощи функций ExamineActions() и NextAction().
;   Атрибуты текущего действия можно получить при помощи функций ActionIndex(), ActionName() и ActionShortcutString().
; - Из модуля Actions за ненадобностью удалены функции UnBindMenuAction(), GetActionShortcutString() и FillListAction().
; - Добавлен модуль ExtTools для поддержки запуска внешних инструментов

; IDE Options = PureBasic 5.51 (Windows - x86)
; CursorPosition = 290
; FirstLine = 266
; Folding = ---
; EnableXP